@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB Inc.
+ * @copyright Copyright (c) 2017-2023 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -76,7 +76,6 @@ struct DenseNegFx {
   void read_dense_array_global(const std::string& path);
   void read_dense_array_row(const std::string& path);
   void read_dense_array_col(const std::string& path);
-  static std::string random_name(const std::string& prefix);
 };
 
 DenseNegFx::DenseNegFx()
@@ -104,13 +103,6 @@ void DenseNegFx::remove_temp_dir(const std::string& path) {
     REQUIRE(tiledb_vfs_remove_dir(ctx_, vfs_, path.c_str()) == TILEDB_OK);
 }
 
-std::string DenseNegFx::random_name(const std::string& prefix) {
-  std::stringstream ss;
-  ss << prefix << "-" << std::this_thread::get_id() << "-"
-     << TILEDB_TIMESTAMP_NOW_MS;
-  return ss.str();
-}
-
 void DenseNegFx::create_dense_vector(const std::string& path) {
   int rc;
   int64_t dim_domain[] = {-1, 2};
