@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB Inc.
+ * @copyright Copyright (c) 2017-2023 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -118,7 +118,6 @@ struct SparseHeterFx {
       const std::vector<int64_t>& buff_d1,
       const std::vector<float>& buff_d2,
       const std::vector<int32_t>& buff_a);
-  static std::string random_name(const std::string& prefix);
   int tiledb_array_get_non_empty_domain_from_index_wrapper(
       tiledb_ctx_t* ctx,
       tiledb_array_t* array,
@@ -322,13 +321,6 @@ void SparseHeterFx::remove_temp_dir(const std::string& path) {
     REQUIRE(tiledb_vfs_remove_dir(ctx_, vfs_, path.c_str()) == TILEDB_OK);
 }
 
-std::string SparseHeterFx::random_name(const std::string& prefix) {
-  std::stringstream ss;
-  ss << prefix << "-" << std::this_thread::get_id() << "-"
-     << TILEDB_TIMESTAMP_NOW_MS;
-  return ss.str();
-}
-
 void SparseHeterFx::check_non_empty_domain_float_int64(
     const std::string& path,
     const float* dom_f,
@@ -988,9 +980,22 @@ TEST_CASE_METHOD(
 
   // ####### CONSOLIDATE #######
 
-  int rc = tiledb_array_consolidate(ctx_, array_name.c_str(), nullptr);
+  // Consolidate fragments
+  tiledb_config_t* config = nullptr;
+  tiledb_error_t* error = nullptr;
+  REQUIRE(tiledb_config_alloc(&config, &error) == TILEDB_OK);
+  REQUIRE(error == nullptr);
+
+  auto rc = tiledb_config_set(
+      config, "sm.consolidation.total_buffer_size", "1048576", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+
+  rc = tiledb_array_consolidate(ctx_, array_name.c_str(), config);
   CHECK(rc == TILEDB_OK);
 
+  tiledb_config_free(&config);
+
   // Check non-empty domain
   c_dom_f = {1.1f, 1.5f};
   c_dom_i = {1, 6};
@@ -1342,9 +1347,22 @@ TEST_CASE_METHOD(
 
   // ####### CONSOLIDATE #######
 
-  int rc = tiledb_array_consolidate(ctx_, array_name.c_str(), nullptr);
+  // Consolidate fragments
+  tiledb_config_t* config = nullptr;
+  tiledb_error_t* error = nullptr;
+  REQUIRE(tiledb_config_alloc(&config, &error) == TILEDB_OK);
+  REQUIRE(error == nullptr);
+
+  int rc = tiledb_config_set(
+      config, "sm.consolidation.total_buffer_size", "1048576", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+
+  rc = tiledb_array_consolidate(ctx_, array_name.c_str(), config);
   CHECK(rc == TILEDB_OK);
 
+  tiledb_config_free(&config);
+
   // Check non-empty domain
   c_dom_i = {1, 6};
   c_dom_f = {1.1f, 1.5f};
