@@ -269,6 +269,9 @@ class Config {
   /** The buffer size for each attribute used in consolidation. */
   static const std::string SM_CONSOLIDATION_BUFFER_SIZE;
 
+  /** The total buffer size for all attributes during consolidation. */
+  static const std::string SM_CONSOLIDATION_TOTAL_BUFFER_SIZE;
+
   /** The maximum fragment size used in consolidation. */
   static const std::string SM_CONSOLIDATION_MAX_FRAGMENT_SIZE;
 
@@ -565,6 +568,12 @@ class Config {
   /** Force making an unsigned request to s3 (false). */
   static const std::string VFS_S3_NO_SIGN_REQUEST;
 
+  /**
+   * When set to `true`, the S3 SDK uses a handler that ignores SIGPIPE
+   * signals.
+   */
+  static const std::string VFS_S3_INSTALL_SIGPIPE_HANDLER;
+
   /** HDFS default kerb ticket cache path. */
   static const std::string VFS_HDFS_KERB_TICKET_CACHE_PATH;
 
