@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2023 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -101,22 +101,8 @@ Status GroupMetaConsolidator::consolidate(
   // Metadata uris to delete
   const auto to_vacuum = metadata_w->loaded_metadata_uris();
 
-  // Generate new name for consolidated metadata
-  st = metadata_w->generate_uri(group_uri);
-  if (!st.ok()) {
-    throw_if_not_ok(group_for_reads.close());
-    throw_if_not_ok(group_for_writes.close());
-    return st;
-  }
-
-  // Get the new URI name
-  URI new_uri;
-  st = metadata_w->get_uri(group_uri, &new_uri);
-  if (!st.ok()) {
-    throw_if_not_ok(group_for_reads.close());
-    throw_if_not_ok(group_for_writes.close());
-    return st;
-  }
+  // Get the new URI name for consolidated metadata
+  URI new_uri = metadata_w->get_uri(group_uri);
 
   // Close groups
   RETURN_NOT_OK_ELSE(
