@@ -91,11 +91,12 @@ class ResultTile {
         std::string name,
         const bool var_size,
         const bool nullable,
+        const bool validity_only,
         const uint64_t tile_idx)
-        : tile_size_(fragment->tile_size(name, tile_idx))
+        : tile_size_(validity_only ? 0 : fragment->tile_size(name, tile_idx))
         , tile_persisted_size_(fragment->persisted_tile_size(name, tile_idx))
         , tile_var_size_(
-              var_size ?
+              var_size && !validity_only ?
                   std::optional(fragment->tile_var_size(name, tile_idx)) :
                   std::nullopt)
         , tile_var_persisted_size_(
