@@ -50,6 +50,7 @@
 #include "tiledb/sm/serialization/array_directory.h"
 #include "tiledb/sm/serialization/array_schema.h"
 #include "tiledb/sm/serialization/fragment_metadata.h"
+#include "tiledb/sm/storage_manager/storage_manager.h"
 
 using namespace tiledb::common;
 using namespace tiledb::sm::stats;
@@ -239,33 +240,26 @@ Status array_from_capnp(
   if (array_reader.hasUri()) {
     array->set_uri_serialized(array_reader.getUri().cStr());
   }
-  array->set_timestamp_start(array_reader.getStartTimestamp());
-  array->set_timestamp_end(array_reader.getEndTimestamp());
 
   if (array_reader.hasQueryType()) {
     auto query_type_str = array_reader.getQueryType();
     QueryType query_type = QueryType::READ;
     RETURN_NOT_OK(query_type_enum(query_type_str, &query_type));
     array->set_query_type(query_type);
-    if (!array->is_open()) {
-      array->set_serialized_array_open();
-    }
 
-    array->set_timestamp_end_opened_at(array_reader.getOpenedAtEndTimestamp());
-    if (array->timestamp_end_opened_at() == UINT64_MAX) {
-      if (query_type == QueryType::READ) {
-        array->set_timestamp_end_opened_at(
-            tiledb::sm::utils::time::timestamp_now_ms());
-      } else if (
-          query_type == QueryType::WRITE ||
-          query_type == QueryType::MODIFY_EXCLUSIVE ||
-          query_type == QueryType::DELETE || query_type == QueryType::UPDATE) {
-        array->set_timestamp_end_opened_at(0);
-      } else {
-        throw StatusException(Status_SerializationError(
-            "Cannot open array; Unsupported query type."));
-      }
-    }
+    array->set_timestamps(
+        array_reader.getStartTimestamp(),
+        array_reader.getEndTimestamp(),
+        query_type == QueryType::READ);
+  } else {
+    array->set_timestamps(
+        array_reader.getStartTimestamp(),
+        array_reader.getEndTimestamp(),
+        false);
+  };
+
+  if (!array->is_open()) {
+    array->set_serialized_array_open();
   }
 
   if (array_reader.hasArraySchemasAll()) {
@@ -282,7 +276,7 @@ Status array_from_capnp(
             make_shared<ArraySchema>(HERE(), schema);
       }
     }
-    array->set_array_schemas_all(all_schemas);
+    array->set_array_schemas_all(std::move(all_schemas));
   }
 
   if (array_reader.hasArraySchemaLatest()) {
@@ -301,26 +295,31 @@ Status array_from_capnp(
         array_directory_reader,
         storage_manager->resources(),
         array->array_uri());
-    array->set_array_directory(*array_dir);
+    array->set_array_directory(std::move(*array_dir));
+  } else {
+    array->set_array_directory(
+        ArrayDirectory(storage_manager->resources(), array->array_uri()));
   }
 
   if (array_reader.hasFragmentMetadataAll()) {
-    array->fragment_metadata().clear();
-    array->fragment_metadata().reserve(
-        array_reader.getFragmentMetadataAll().size());
-    for (auto frag_meta_reader : array_reader.getFragmentMetadataAll()) {
+    auto fragment_metadata = array->fragment_metadata();
+    fragment_metadata.clear();
+    auto fragment_metadata_all_reader = array_reader.getFragmentMetadataAll();
+    fragment_metadata.reserve(fragment_metadata_all_reader.size());
+    for (auto frag_meta_reader : fragment_metadata_all_reader) {
       auto meta = make_shared<FragmentMetadata>(HERE());
       RETURN_NOT_OK(fragment_metadata_from_capnp(
           array->array_schema_latest_ptr(),
           frag_meta_reader,
           meta,
-          storage_manager,
+          &storage_manager->resources(),
           array->memory_tracker()));
       if (client_side) {
         meta->set_rtree_loaded();
       }
-      array->fragment_metadata().emplace_back(meta);
+      fragment_metadata.emplace_back(meta);
     }
+    array->set_fragment_metadata(std::move(fragment_metadata));
   }
 
   if (array_reader.hasNonEmptyDomain()) {
@@ -563,11 +562,21 @@ Status array_deserialize(
         break;
       }
       case SerializationType::CAPNP: {
+        // Set traversal limit from config
+        uint64_t limit = storage_manager->config()
+                             .get<uint64_t>("rest.capnp_traversal_limit")
+                             .value();
+        ::capnp::ReaderOptions readerOptions;
+        // capnp uses the limit in words
+        readerOptions.traversalLimitInWords = limit / sizeof(::capnp::word);
+
         const auto mBytes =
             reinterpret_cast<const kj::byte*>(serialized_buffer.data());
-        ::capnp::FlatArrayMessageReader reader(kj::arrayPtr(
-            reinterpret_cast<const ::capnp::word*>(mBytes),
-            serialized_buffer.size() / sizeof(::capnp::word)));
+        ::capnp::FlatArrayMessageReader reader(
+            kj::arrayPtr(
+                reinterpret_cast<const ::capnp::word*>(mBytes),
+                serialized_buffer.size() / sizeof(::capnp::word)),
+            readerOptions);
         capnp::Array::Reader array_reader = reader.getRoot<capnp::Array>();
         RETURN_NOT_OK(array_from_capnp(array_reader, storage_manager, array));
         break;
