@@ -69,6 +69,7 @@ namespace sm {
 
 class Array;
 class ArraySchema;
+class OpenedArray;
 class DimensionLabel;
 class EncryptionKey;
 class FragIdx;
@@ -247,6 +248,26 @@ class Subarray {
       bool coalesce_ranges = true,
       StorageManager* storage_manager = nullptr);
 
+  /**
+   * Constructor.
+   *
+   * @param opened_array The opened array the subarray is associated with.
+   * @param layout The layout of the values of the subarray (of the results
+   *     if the subarray is used for reads, or of the values provided
+   *     by the user for writes).
+   * @param parent_stats The parent stats to inherit from.
+   * @param logger The parent logger to clone and use for logging
+   * @param coalesce_ranges When enabled, ranges will attempt to coalesce
+   *     with existing ranges as they are added.
+   */
+  Subarray(
+      const shared_ptr<OpenedArray> opened_array,
+      Layout layout,
+      stats::Stats* parent_stats,
+      shared_ptr<Logger> logger,
+      bool coalesce_ranges = true,
+      StorageManager* storage_manager = nullptr);
+
   /**
    * Copy constructor. This performs a deep copy (including memcpy of
    * underlying buffers).
@@ -273,7 +294,7 @@ class Subarray {
   /* ********************************* */
 
   /** Sets config for query-level parameters only. */
-  void set_config(const Config& config);
+  void set_config(const QueryType query_type, const Config& config);
 
   /**
    * Get the config of the writer
@@ -593,8 +614,8 @@ class Subarray {
       void* start,
       void* end) const;
 
-  /** Returns the array the subarray is associated with. */
-  const Array* array() const;
+  /** Returns the opened array the subarray is associated with. */
+  const shared_ptr<OpenedArray> array() const;
 
   /**
    * Returns the number of cells in the subarray.
@@ -1388,7 +1409,7 @@ class Subarray {
   shared_ptr<Logger> logger_;
 
   /** The array the subarray object is associated with. */
-  const Array* array_;
+  shared_ptr<OpenedArray> array_;
 
   /** Stores the estimated result size for each array attribute/dimension. */
   std::unordered_map<std::string, ResultSize> est_result_size_;
